package lc.esp.cli.sdk;

import lc.esp.sdk.*;
import lc.mecha.lang.FutureResult;
import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;
import lc.mecha.util.BasicallyDangerous;

import javax.jms.JMSException;
import java.util.List;

/**
 * This class implements a simple client for sending requests to a {@link CLIService}.
 *
 * @author Alex Leigh
 * @since 18.0
 */
public class CLIClient extends BasicallyDangerous {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(CLIClient.class);
    private final ESPSession session;
    private final ESPRequestor requestor;

    public CLIClient(ESPClient client) throws JMSException {
        this.session = client.createSession();
        this.requestor = session.createRequestor();
    }

    public CLIResponse exec(ESPAddress destination, List<String> cmd) throws Exception {
        logger.info("Received exec(). [cmd: {}]", cmd);

        CLIRequest req = new CLIRequest(cmd);
        ESPMessage reqMsg = req.toMessage();
        FutureResult<ESPMessage> future = requestor.request(destination, reqMsg);
        logger.info("Waiting for reply...");
        return new CLIResponse((ESPMessage) future.get());
    }

    @Override
    public void runDangerously() throws Exception {
        requestor.runDangerously();
    }
}
