package lc.esp.engine.service;

import lc.esp.sdk.ESPAddress;
import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;
import lc.mecha.util.BasicallyDangerous;
import lc.mecha.util.UniversalJob;
import org.apache.activemq.broker.BrokerPlugin;
import org.apache.activemq.broker.BrokerService;
import org.apache.activemq.broker.region.DestinationInterceptor;
import org.apache.activemq.broker.region.virtual.VirtualDestination;
import org.apache.activemq.broker.region.virtual.VirtualDestinationInterceptor;
import org.apache.activemq.broker.region.virtual.VirtualTopic;
import org.apache.activemq.usage.SystemUsage;

import static java.lang.Thread.sleep;

public class EngineService extends BasicallyDangerous {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(EngineService.class);

    public static void main(String args[]) throws InterruptedException {
        UniversalJob.banner(logger, "Enhanced Services Platform mk18/ENGINE (GIPSY DANGER)");

        EngineService eng = new EngineService();
        eng.run();

        //noinspection InfiniteLoopStatement
        while (true) {
            //noinspection BusyWait
            sleep(1000);
        }
    }

    @Override
    public void runDangerously() throws Exception {
        BrokerService broker = new BrokerService();
        broker.addConnector("tcp://0.0.0.0:52018");
        broker.addConnector("mqtt://0.0.0.0:52019");
        broker.addConnector("stomp://0.0.0.0:52020");
        broker.addConnector("ws://0.0.0.0:52021");
        broker.setPersistent(true);

        VirtualDestinationInterceptor virtualDestinationInterceptor = new VirtualDestinationInterceptor();
        VirtualTopic vTopic = new VirtualTopic();
        vTopic.setName(">");
        vTopic.setPrefix(ESPAddress.VIRTUAL_TOPIC_PREFIX + ".*.");
        vTopic.setLocal(false);
        // Note ramifications: https://activemq.apache.org/virtual-destinations
        vTopic.setSelectorAware(true);
        virtualDestinationInterceptor.setVirtualDestinations(new VirtualDestination[]{vTopic});
        DestinationInterceptor[] destinationInterceptors = new DestinationInterceptor[]{virtualDestinationInterceptor};
        broker.setDestinationInterceptors(destinationInterceptors);
        broker.setPlugins(new BrokerPlugin[]{new EnhancedBrokerPlugin()});
        new EnhancedBrokerPlugin().installPlugin(broker.getBroker());
        broker.start();

        //noinspection InfiniteLoopStatement
        while (true) {
            SystemUsage su = broker.getConsumerSystemUsage();
            logger.info("{}", su.getMemoryUsage());
            logger.info("{}", su.getStoreUsage());
            logger.info("{}", su.getTempUsage());
            sleep(60 * 1000);
        }
    }
}